from pygame.mixer import SoundType
from pygalaga.utils import load_sound

class Sounds:
    def __init__(self):
        self.shoot: SoundType = load_sound("shoot.mp3")
        self.explosion: SoundType = load_sound("explosion.mp3")


import pygame
from pygame import SurfaceType

from pygalaga.utils import load_asset
from .sounds import Sounds
from .bullet import Bullet
from pygalaga.config import WIDTH, HEIGHT


class Ship:
    def __init__(self):
        self.image = load_asset("ship.png", (50, 50))
        self.x = WIDTH // 2
        self.y = HEIGHT - 70
        self.speed = 5
        self.bullets = []
        self.last_shot_time = 0
        self.shoot_delay = 500
        self.sounds = Sounds()

    def move(self, direction) -> None:
        if direction == "left" and self.x > 0:
            self.x -= self.speed
        if direction == "right" and self.x < WIDTH - 50:
            self.x += self.speed

    def shoot(self) -> None:
        current_time = pygame.time.get_ticks()
        if current_time - self.last_shot_time >= self.shoot_delay:
            self.bullets.append(Bullet(self.x + 20, self.y))
            self.last_shot_time = current_time
            self.sounds.shoot.play()

    def draw(self, screen: SurfaceType) -> None:
        screen.blit(self.image, (self.x, self.y))
        for bullet in self.bullets:
            bullet.draw(screen)





from pygalaga.utils import load_asset


class Bullet:
    def __init__(self, x, y) -> None:
        self.image = load_asset("missile.png", (10, 20))
        self.x = x
        self.y = y
        self.speed = 7

    def move(self) -> None:
        self.y -= self.speed

    def draw(self, screen) -> None:
        screen.blit(self.image, (self.x, self.y))



import os
import pygame
from functools import cache
from pygame.mixer import SoundType
from PIL import Image


@cache
def load_asset(asset: str, size: tuple[int,int]):
    asset_img = pygame.image.load(get_asset_path(asset))
    return pygame.transform.scale(asset_img, size)

def load_sound(asset:str) -> SoundType:
    return pygame.mixer.Sound(get_sound_path(asset))

@cache
def load_gif_asset(asset: str, size: tuple[int,int]):
    explosion_gif = Image.open(get_asset_path(asset))
    frames = []
    for frame in range(explosion_gif.n_frames):
        explosion_gif.seek(frame)
        frame_image = explosion_gif.convert("RGBA")
        mode = frame_image.mode
        frame_size = frame_image.size
        data = frame_image.tobytes()
        pygame_image = pygame.image.fromstring(data, frame_size, mode)
        frames.append(pygame.transform.scale(pygame_image, size))
    return frames

def get_asset_path(filename):
    return os.path.join(os.path.dirname(__file__), "assets", filename)

def get_sound_path(filename):
    return os.path.join(os.path.dirname(__file__), "assets/sounds", filename)


import random
import pygame
from pygame import SurfaceType

from pygalaga.config import *
from pygalaga.components import Ship, AlienRed, Sounds


def main():
    pygame.init()

    screen: SurfaceType = pygame.display.set_mode((WIDTH, HEIGHT))
    pygame.display.set_caption("Galaga")

    player = Ship()
    enemies = [AlienRed(random.randint(0, WIDTH - 40), random.randint(-100, -40)) for _ in range(2)]
    running = True
    clock = pygame.time.Clock()
    sounds = Sounds()

    while running:
        screen.fill((0, 0, 0))  # Clear screen
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False

        keys = pygame.key.get_pressed()
        if keys[pygame.K_LEFT]:
            player.move("left")
        if keys[pygame.K_RIGHT]:
            player.move("right")
        if keys[pygame.K_SPACE] or keys[pygame.K_UP]:
            player.shoot()

        # Move and draw bullets
        for bullet in player.bullets[:]:
            bullet.move()
            if bullet.y < 0:
                player.bullets.remove(bullet)

        # Move and draw enemies
        for enemy in enemies[:]:
            enemy.move()
            if enemy.y > HEIGHT:
                enemies.remove(enemy)
                enemies.append(AlienRed(random.randint(0, WIDTH - 40), random.randint(-100, -40)))

        # Collision detection
        for bullet in player.bullets[:]:
            for enemy in enemies[:]:
                if enemy.x < bullet.x < enemy.x + 40 and enemy.y < bullet.y < enemy.y + 40:
                    player.bullets.remove(bullet)
                    enemy.exploding = True
                    enemy.explosion_time = pygame.time.get_ticks()
                    sounds.explosion.play()

        for enemy in enemies[:]:
            if enemy.exploding and pygame.time.get_ticks() - enemy.explosion_time > 500:
                enemies.remove(enemy)
                enemies.append(AlienRed(random.randint(0, WIDTH - 40), random.randint(-100, -40)))

        # Draw objects
        player.draw(screen)
        for enemy in enemies:
            enemy.draw(screen)

        pygame.display.flip()
        clock.tick(30)

    pygame.quit()

if __name__ == "__main__":
    main()



from pygalaga.utils import load_asset, load_gif_asset


class AlienRed:
    def __init__(self, x, y) -> None:
        self.image = load_asset("space_invader_red.png", (40, 40))
        self.explosion = load_gif_asset("explosion.gif", (80, 80))
        self.x = x
        self.y = y
        self.speed = 2
        self.__exploding = False
        self.explosion_time = 0
        self.explosion_index = 0

    @property
    def exploding(self):
        return self.__exploding

    @exploding.setter
    def exploding(self, val):
        self.explosion_time = 0
        self.explosion_index = 0
        self.__exploding = val

    def move(self) -> None:
        if not self.exploding: self.y += self.speed

    def draw(self, screen) -> None:
        if self.exploding:
            if self.explosion_index < len(self.explosion):
                screen.blit(self.explosion[self.explosion_index], (self.x-20, self.y-20))
                self.explosion_index += 1
            else:
                self.exploding = False
                self.explosion_index = 0
        else:
            screen.blit(self.image, (self.x, self.y))
